#include "about.h"
#include "ui_about.h"

#ifdef USE_WEBKIT
#include <QWebView>
#else
#include <QTextEdit>
#include <QFile>
#include <QTextStream>
#endif

About *About::pInstance = NULL;

About *About::instance()
{
	if (pInstance)
		return pInstance;
	return new About;
}

void About::deleteInstance()
{
	if (pInstance)
		pInstance->deleteLater();
}

About::About(QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::About)
{
	pInstance = this;

    m_ui->setupUi(this);
	m_ui->version->setText(QApplication::applicationVersion());

#ifdef USE_WEBKIT
	m_ui->tab_about->layout()->addWidget(about = new QWebView);
	m_ui->tab_license->layout()->addWidget(license = new QWebView);
	m_ui->tab_credits->layout()->addWidget(credits = new QWebView);
#else
	m_ui->tab_about->layout()->addWidget(about = new QTextEdit);
	m_ui->tab_license->layout()->addWidget(license = new QTextEdit);
	m_ui->tab_credits->layout()->addWidget(credits = new QTextEdit);
#endif
	loadPages();
}

About::~About()
{
    delete m_ui;

	pInstance = NULL;
}

void About::hideEvent(QHideEvent *)
{
	deleteLater();
}

void About::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
		loadPages();
		break;
    default:
        break;
    }
}

void About::loadPages()
{
#ifdef USE_WEBKIT
	about->load(QUrl(QString(DOCS_PATH) + tr("qshare.html")));
	license->load(QUrl(QString(DOCS_PATH) + tr("gpl.html")));
	credits->load(QUrl(QString(DOCS_PATH) + tr("credits.html")));
#else
	QFile f_about(QString(DOCS_PATH) + tr("qshare.html"));
	QFile f_license(QString(DOCS_PATH) + tr("gpl.html"));
	QFile f_credits(QString(DOCS_PATH) + tr("credits.html"));
	f_about.open(QIODevice::ReadOnly);
	f_license.open(QIODevice::ReadOnly);
	f_credits.open(QIODevice::ReadOnly);
	QTextStream ts_about(&f_about);
	QTextStream ts_license(&f_license);
	QTextStream ts_credits(&f_credits);
	ts_about.setCodec("UTF-8");
	ts_license.setCodec("UTF-8");
	ts_credits.setCodec("UTF-8");
	about->setHtml(ts_about.readAll());
	license->setHtml(ts_license.readAll());
	credits->setHtml(ts_credits.readAll());
#endif
}
