#ifndef CONFIG_H
#define CONFIG_H

#include <QtGui/QDialog>
#include <QMutex>
#include "userid.h"
#include "sharedfolders.h"

class QTableWidgetItem;
class QAbstractButton;
class QUrl;

namespace Ui {
    class Config;
}

class Config : public QDialog {
    Q_OBJECT
public:
    Config(QWidget *parent = 0);
    ~Config();

	static bool isZeroconfEnabled();
	static void setZeroconfEnabled(bool enabled);
	static bool isAnonymous();
	static bool isAnonymousReadAllowed();
	static bool showHiddenFiles();
	static bool checkPassword(const QString &passwd);
	static QString getLogin();
	static QString getHostname();
	static int getServerPort();
	static bool checkID(const UserID &uid, const bool read = false);
	static UserID getUID();

public slots:
	void updateServerStatus();
	void saveSettings();
	void addFolder();
	void removeFolder();
	void editFolder(QTableWidgetItem *item);
	void editPath(QTableWidgetItem *item);
	void loadFolders();
	void changeLanguage();
	void loadLanguages();
	void openURL(const QUrl &url);
	void addPath(const bool writable, const QString &path);
	void removePath(const QString &path);

private slots:
	void processButtonClick(QAbstractButton *);
	void restoreFolders();

protected:
    void changeEvent(QEvent *e);
	void loadSettings();
private:
    Ui::Config *m_ui;
	QList<QString> folderNames;
	QMap<QString, SharedFolders::Folder> foldersBackup;

private:
	static Config *pInstance;
	static QMutex sMutex;
public:
	static Config *instance();
	static void deleteInstance();
	static void lock();
	static void unlock();
};

#endif // CONFIG_H
