#ifndef FTPSERVER_H
#define FTPSERVER_H

#include <QList>
#include <QSet>
#include "ipobject.h"
#include "userid.h"

class QTcpServer;

class FtpServer : public IPObject
{
	Q_OBJECT

	friend class RETRcmd;
	friend class FINDcmd;
	friend class STORcmd;
	friend class LISTcmd;
public:
	struct ConnectionInfo
	{
		QString user;
		QString address;
		QString activity;
	};
public:
	FtpServer();
	virtual ~FtpServer();
	QList<ConnectionInfo> getInfo();
	const QSet<QString> &getUsers() const {	return users;	}
	int nbConnections();
	bool checkID(const UserID &uid) const	{	return validIDs.contains(uid);	}
	bool knownLogin(const QString &login) const;

protected:
	virtual void receive(const QByteArray &message);

private slots:
	void addConnection();
	void statsSend(int);
	void statsRecv(int);
	void killThread();
	void startService();
	void broadcastNewThread();
	void broadcastRemoveThread();
	void removeClient();

signals:
	void newThread();
	void removeThread();
	void closing();

private:
	QTcpServer *sock;
	// For statistics
	quint64 totalSent;
	quint64 totalReceived;
	int maxConnections;
	int totalConnections;
	QSet<QString> users;
	QSet<UserID> validIDs;

private:
	static FtpServer *pInstance;
public:
	static void startServer();
	static void stopServer();
	static void restartServer();
	static bool serverRunning();
	static QList<ConnectionInfo> getClientsInfo();
	static QStringList getServerInfo();
	static FtpServer *instance();
};

#endif // FTPSERVER_H
