#include "path.h"
#include <QDateTime>
#include "sharedfolders.h"
#include "ftpserver.h"
#include "userspath.h"
#include <QDebug>

const QString Path::separator('/');

Path::Path()
{
	currentPath = "/";
}

QString Path::pwd()
{
	return currentPath;
}

bool Path::cwd(const QString &path)
{
	const QString absolutePath = absolute(path);
	if (!UsersPath::isDir(uid, absolutePath) || !UsersPath::exists(uid, absolutePath))
		return false;

	currentPath = absolutePath;
	return true;
}

bool Path::mkd(const QString &name)
{
	const QString absolutePath = absolute(name);
	return UsersPath::mkd(uid, absolutePath);
}

void Path::chmod(const QString &path, QByteArray perms)
{
	const QString absolutePath = absolute(path);
	return UsersPath::chmod(uid, absolutePath, perms);
}

bool Path::rmd(const QString &name)
{
	const QString absolutePath = absolute(name);
	return UsersPath::rmd(uid, absolutePath);
}

bool Path::dele(const QString &name)
{
	const QString absolutePath = absolute(name);
	return UsersPath::dele(uid, absolutePath);
}

bool Path::cdup()
{
	if (currentPath == separator)
		return false;
	currentPath = SharedFolders::getParentPath(currentPath);
	return true;
}

bool Path::rename(const QString &from, const QString &to)
{
	const QString absolutePathFrom = absolute(from);
	const QString absolutePathTo = absolute(to);
	return UsersPath::rename(uid, absolutePathFrom, absolutePathTo);
}

void Path::list(QList<QByteArray> &sList, const QString &filename, bool utf8)
{
	if (!filename.isEmpty() && filename[0] == '-')
	{
		list(sList, utf8);
		return;
	}
	const QString absolutePath = absolute(filename);
	UsersPath::list(uid, sList, absolutePath, utf8);
}

void Path::list(QList<QByteArray> &sList, bool utf8)
{
	const QString absolutePath = absolute(currentPath);
	UsersPath::list(uid, sList, absolutePath, utf8);
}

void Path::nlst(QList<QByteArray> &sList, bool utf8)
{
	const QString absolutePath = absolute(currentPath);
	UsersPath::nlst(uid, sList, absolutePath, utf8);
}

QString Path::absolute(const QString &path)
{
	QString absolutePath = path;
	if (!path.isEmpty() && path[0] != '/')
		absolutePath = currentPath + separator + path;
	const QStringList keys = absolutePath.split(separator, QString::SkipEmptyParts);
	if (keys.empty())
		return separator;
	QStringList out;
	foreach(QString elt, keys)
	{
		if (elt == "..")
		{
			if (!out.isEmpty())
				out.removeLast();
		}
		else
			out << elt;
	}
	return separator + out.join(separator);
}

QString Path::realPath(const QString &path)
{
	return SharedFolders::instance()->getRealPath(absolute(path));
}

bool Path::exists(const QString &path)
{
	return UsersPath::exists(uid, absolute(path));
}

qint64 Path::size(const QString &name)
{
	return UsersPath::size(uid, absolute(name));
}

QDateTime Path::lastModified(const QString &name)
{
	return UsersPath::lastModified(uid, absolute(name));
}

bool Path::isReadable(const QString &name)
{
	return UsersPath::isReadable(uid, absolute(name));
}

bool Path::isWritable(const QString &name)
{
	return UsersPath::isWritable(uid, absolute(name));
}

void Path::SetUID(const UserID &uid)
{
	this->uid = uid;
}
