#ifndef SHAREDFOLDERS_H
#define SHAREDFOLDERS_H

#include <QObject>
#include <QMutex>
#include <QDir>
#include <QMap>

class SharedFolders : public QObject, public QMutex
{
	Q_OBJECT
public:
	class Folder
	{
		friend class SharedFolders;
	public:
		Folder(const QString &path = QString(), bool writable = false);

		bool isWritable() const;
		const QString &getPath() const;

		void setWritable(bool writable);
		void setPath(const QString &path);
	private:
		bool writable;
		QString path;
	};

public:
    SharedFolders();
	~SharedFolders();
	QMap<QString, Folder> getFolderList();
	QString getRealPath(const QString &path);
	bool isWritable(const QString &path);
	int nbFolders();
	static QString getParentPath(const QString &path);
	static QString getFileName(const QString &path);

public slots:
	void addFolder(const QString &name, const Folder &folder);
	void removeFolder(const QString &name);

private:
	void loadFolders();
	void saveFolders();

private:
	QMap<QString, Folder>	folders;

private:
	static SharedFolders *pInstance;
public:
	static SharedFolders *instance();
	static const QString separator;
};

#endif // SHAREDFOLDERS_H
